@@ -40,12 +40,14 @@
 #include <gtk/gtkdnd.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkfilesel.h>
+#include <gtk/gtkfilechooser.h>
 #include <gtk/gtkframe.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkpixmap.h>
 #include <gtk/gtkscrolledwindow.h>
+#include <gtk/gtktreeview.h>
 
 #include "gnome-i18nP.h"
 
@@ -54,12 +56,15 @@
 #include "gnome-pixmap-entry.h"
 #include "gnome-pixmap.h"
 
-#include <libgnomevfs/gnome-vfs-mime.h>
+#include <libgnomevfs/gnome-vfs-uri.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
+
+#include "libgnomeui-access.h"
 
 struct _GnomePixmapEntryPrivate {
 	GtkWidget *preview;
 	GtkWidget *preview_sw;
-	
+
 	gchar *last_preview;
 
 	guint32 do_preview : 1; /*put a preview frame with the pixmap next to
@@ -67,41 +72,30 @@ struct _GnomePixmapEntryPrivate {
 };
 
 
-static void gnome_pixmap_entry_class_init (GnomePixmapEntryClass *class);
-static void gnome_pixmap_entry_instance_init (GnomePixmapEntry      *pentry);
 static void drag_data_get		  (GtkWidget          *widget,
 					   GdkDragContext     *context,
 					   GtkSelectionData   *selection_data,
 					   guint               info,
 					   guint               time,
 					   GnomePixmapEntry   *pentry);
-static void drag_data_received		  (GtkWidget        *widget,
-					   GdkDragContext   *context,
-					   gint              x,
-					   gint              y,
-					   GtkSelectionData *selection_data,
-					   guint             info,
-					   guint32           time,
-					   GnomePixmapEntry *pentry);
 static void pentry_destroy		  (GtkObject *object);
 static void pentry_finalize		  (GObject *object);
-static void pentry_set_arg                (GtkObject *object,
-					   GtkArg *arg,
-					   guint arg_id);
-static void pentry_get_arg                (GtkObject *object,
-					   GtkArg *arg,
-					   guint arg_id);
+
+static void pentry_set_property (GObject *object, guint param_id,
+				 const GValue *value, GParamSpec *pspec);
+static void pentry_get_property (GObject *object, guint param_id,
+				 GValue *value, GParamSpec *pspec);
 
 static GtkTargetEntry drop_types[] = { { "text/uri-list", 0, 0 } };
 
+/* Property IDs */
 enum {
-	ARG_0,
-	ARG_DO_PREVIEW
+	PROP_0,
+	PROP_DO_PREVIEW
 };
 
 GNOME_CLASS_BOILERPLATE (GnomePixmapEntry, gnome_pixmap_entry,
-			 GnomeFileEntry, gnome_file_entry,
-			 GNOME_TYPE_FILE_ENTRY)
+			 GnomeFileEntry, GNOME_TYPE_FILE_ENTRY)
 
 static void
 gnome_pixmap_entry_class_init (GnomePixmapEntryClass *class)
@@ -109,51 +103,60 @@ gnome_pixmap_entry_class_init (GnomePixmapEntryClass *class)
 	GtkObjectClass *object_class = GTK_OBJECT_CLASS(class);
 	GObjectClass *gobject_class = G_OBJECT_CLASS(class);
 
-	gtk_object_add_arg_type("GnomePixmapEntry::do_preview",
-				GTK_TYPE_BOOL,
-				GTK_ARG_READWRITE,
-				ARG_DO_PREVIEW);
+	gobject_class->finalize = pentry_finalize;
+	gobject_class->set_property = pentry_set_property;
+	gobject_class->get_property = pentry_get_property;
+
+	g_object_class_install_property (gobject_class,
+					 PROP_DO_PREVIEW,
+					 g_param_spec_boolean (
+						 "do_preview",
+						 _("Do Preview"),
+						 _("Whether the pixmap entry should have a preview."),
+						 FALSE,
+						 G_PARAM_READWRITE));
 
 	object_class->destroy = pentry_destroy;
-	gobject_class->finalize = pentry_finalize;
-	object_class->get_arg = pentry_get_arg;
-	object_class->set_arg = pentry_set_arg;
-
 }
 
+/* set_property handler for the pixmap entry */
 static void
-pentry_set_arg (GtkObject *object,
-		GtkArg *arg,
-		guint arg_id)
+pentry_set_property (GObject *object, guint param_id,
+		     const GValue *value, GParamSpec *pspec)
 {
-	GnomePixmapEntry *self;
+	GnomePixmapEntry *pentry;
 
-	self = GNOME_PIXMAP_ENTRY (object);
+	pentry = GNOME_PIXMAP_ENTRY (object);
 
-	switch (arg_id) {
-	case ARG_DO_PREVIEW:
-		gnome_pixmap_entry_set_preview(self, GTK_VALUE_BOOL(*arg));
+	switch (param_id) {
+	case PROP_DO_PREVIEW:
+		gnome_pixmap_entry_set_preview(pentry, g_value_get_boolean (value));
 		break;
 
 	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
 	}
 }
 
+/* get_property handler for the pixmap entry */
 static void
-pentry_get_arg (GtkObject *object,
-		GtkArg *arg,
-		guint arg_id)
+pentry_get_property (GObject *object, guint param_id,
+		     GValue *value, GParamSpec *pspec)
 {
-	GnomePixmapEntry *self;
+	GnomePixmapEntry *pentry;
+	GnomePixmapEntryPrivate *priv;
 
-	self = GNOME_PIXMAP_ENTRY (object);
+	pentry = GNOME_PIXMAP_ENTRY (object);
+	priv = pentry->_priv;
 
-	switch (arg_id) {
-	case ARG_DO_PREVIEW:
-		GTK_VALUE_BOOL(*arg) = self->_priv->do_preview;
+	switch (param_id) {
+	case PROP_DO_PREVIEW:
+		g_value_set_boolean (value, priv->do_preview);
 		break;
+
 	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
 	}
 }
@@ -169,7 +172,7 @@ refresh_preview(GnomePixmapEntry *pentry)
 
 	if( ! pentry->_priv->preview)
 		return;
-	
+
 	t = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(pentry), FALSE);
 
 	if(pentry->_priv->last_preview && t
@@ -199,19 +202,23 @@ refresh_preview(GnomePixmapEntry *pentry)
         } else {
 		gtk_widget_destroy(pentry->_priv->preview->parent);
 		pentry->_priv->preview = gtk_image_new_from_pixbuf (pixbuf);
+		_add_atk_name_desc (pentry->_priv->preview,
+				    _("Image Preview"),
+				    _("A preview of the image currently specified"));
+
 		gtk_widget_show(pentry->_priv->preview);
 		gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(pentry->_priv->preview_sw),
 						      pentry->_priv->preview);
 		if(!GTK_WIDGET_NO_WINDOW(pentry->_priv->preview)) {
-			gtk_signal_connect (GTK_OBJECT (pentry->_priv->preview), "drag_data_get",
-					    GTK_SIGNAL_FUNC (drag_data_get),pentry);
+			g_signal_connect (G_OBJECT (pentry->_priv->preview), "drag_data_get",
+					  G_CALLBACK (drag_data_get), pentry);
 			gtk_drag_source_set (pentry->_priv->preview,
 					     GDK_BUTTON1_MASK|GDK_BUTTON3_MASK,
 					     drop_types, 1,
 					     GDK_ACTION_COPY);
 		}
-		gtk_signal_connect (GTK_OBJECT (pentry->_priv->preview->parent), "drag_data_get",
-				    GTK_SIGNAL_FUNC (drag_data_get),pentry);
+		g_signal_connect (G_OBJECT (pentry->_priv->preview->parent), "drag_data_get",
+				  G_CALLBACK (drag_data_get), pentry);
 		gtk_drag_source_set (pentry->_priv->preview->parent,
 				     GDK_BUTTON1_MASK|GDK_BUTTON3_MASK,
 				     drop_types, 1,
@@ -219,7 +226,7 @@ refresh_preview(GnomePixmapEntry *pentry)
 	}
 	g_free(pentry->_priv->last_preview);
 	pentry->_priv->last_preview = t;
-        gdk_pixbuf_unref(pixbuf);
+        g_object_unref (G_OBJECT (pixbuf));
 }
 
 static int change_timeout;
@@ -262,21 +269,28 @@ entry_changed(GtkWidget *w, GnomePixmapEntry *pentry)
 }
 
 static void
-setup_preview(GtkWidget *widget)
+setup_preview(GtkTreeSelection *selection, GtkWidget *widget)
 {
 	const char *p;
 	GList *l;
 	GtkWidget *pp = NULL;
         GdkPixbuf *pixbuf, *scaled;
 	int w,h;
-	GtkWidget *frame;
-	GtkFileSelection *fs;
+	GtkWidget *frame, *fw;
 
-	g_return_if_fail (widget != NULL);
 	g_return_if_fail (GTK_IS_WIDGET (widget));
 
-	frame = gtk_object_get_data(GTK_OBJECT(widget),"frame");
-	fs = gtk_object_get_data(GTK_OBJECT(frame),"fs");
+	if (GTK_IS_FILE_CHOOSER (widget)) {
+		fw = widget;
+		frame = gtk_file_chooser_get_preview_widget (GTK_FILE_CHOOSER (fw));
+
+		p = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER (fw));
+	} else {
+		frame = g_object_get_data (G_OBJECT (widget), "frame");
+		fw = g_object_get_data (G_OBJECT (frame), "fs");
+
+		p = gtk_file_selection_get_filename(GTK_FILE_SELECTION (fw));
+	}
 
 	if((l = gtk_container_get_children(GTK_CONTAINER(frame))) != NULL) {
 		pp = l->data;
@@ -286,7 +300,6 @@ setup_preview(GtkWidget *widget)
 	if(pp)
 		gtk_widget_destroy(pp);
 
-	p = gtk_file_selection_get_filename(fs);
 	if(!p || !g_file_test(p,G_FILE_TEST_IS_SYMLINK|G_FILE_TEST_IS_REGULAR) ||
 	   !(pixbuf = gdk_pixbuf_new_from_file (p, NULL)))
 		return;
@@ -306,18 +319,17 @@ setup_preview(GtkWidget *widget)
 	}
 	scaled = gdk_pixbuf_scale_simple
 		(pixbuf, w, h, GDK_INTERP_BILINEAR);
-        gdk_pixbuf_unref (pixbuf);
+        g_object_unref (G_OBJECT (pixbuf));
 	pp = gtk_image_new_from_pixbuf (scaled);
-        gdk_pixbuf_unref (scaled);
+        g_object_unref (G_OBJECT (scaled));
 
 	gtk_widget_show(pp);
 
 	gtk_container_add(GTK_CONTAINER(frame),pp);
-
 }
 
 static void
-pentry_destroy(GtkObject *object)
+pentry_destroy (GtkObject *object)
 {
 	GnomePixmapEntry *pentry;
 
@@ -326,43 +338,39 @@ pentry_destroy(GtkObject *object)
 
 	/* remember, destroy can be run multiple times! */
 
-	pentry = GNOME_PIXMAP_ENTRY(object);
+	pentry = GNOME_PIXMAP_ENTRY (object);
 
-	g_free(pentry->_priv->last_preview);
-	pentry->_priv->last_preview = NULL;
-
-	changed_pentries = g_slist_remove(changed_pentries, pentry);
-
-	if(GTK_OBJECT_CLASS(parent_class)->destroy)
-		(* GTK_OBJECT_CLASS(parent_class)->destroy)(object);
+	changed_pentries = g_slist_remove (changed_pentries, pentry);
 
+	GNOME_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
 static void
-pentry_finalize(GObject *object)
+pentry_finalize (GObject *object)
 {
 	GnomePixmapEntry *pentry;
 
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (GNOME_IS_PIXMAP_ENTRY (object));
 
-	pentry = GNOME_PIXMAP_ENTRY(object);
+	pentry = GNOME_PIXMAP_ENTRY (object);
 
-	g_free(pentry->_priv);
+	g_free (pentry->_priv->last_preview);
+	pentry->_priv->last_preview = NULL;
+
+	g_free (pentry->_priv);
 	pentry->_priv = NULL;
 
-	if(G_OBJECT_CLASS(parent_class)->finalize)
-		(* G_OBJECT_CLASS(parent_class)->finalize)(object);
-
+	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
 static void
 browse_clicked(GnomeFileEntry *fentry, GnomePixmapEntry *pentry)
 {
-	GtkWidget *w;
-	GtkWidget *hbox;
-
+	GtkWidget *w, *hbox;
 	GtkFileSelection *fs;
+	GtkFileChooser *fc;
+	gchar *path;
 
 	g_return_if_fail (fentry != NULL);
 	g_return_if_fail (GNOME_IS_FILE_ENTRY (fentry));
@@ -371,77 +379,57 @@ browse_clicked(GnomeFileEntry *fentry, GnomePixmapEntry *pentry)
 
 	if(!fentry->fsw)
 		return;
-	fs = GTK_FILE_SELECTION(fentry->fsw);
 
-	hbox = fs->file_list;
-	do {
-		hbox = hbox->parent;
-		if(!hbox) {
-			g_warning(_("Can't find an hbox, using a normal file "
-				    "selection"));
+	if (GTK_IS_FILE_CHOOSER (fentry->fsw)) {
+		fc = GTK_FILE_CHOOSER (fentry->fsw);
+
+		if (gtk_file_chooser_get_preview_widget (fc) != NULL)
 			return;
-		}
-	} while(!GTK_IS_HBOX(hbox));
 
-	w = gtk_frame_new(_("Preview"));
-	gtk_widget_show(w);
-	gtk_box_pack_end(GTK_BOX(hbox),w,FALSE,FALSE,0);
-	gtk_widget_set_usize(w,110,110);
-	gtk_object_set_data(GTK_OBJECT(w),"fs",fs);
+		w = gtk_frame_new ("");
+		gtk_frame_set_shadow_type (GTK_FRAME (w), GTK_SHADOW_NONE);
+		gtk_widget_show (w);
+		
+		gtk_file_chooser_set_preview_widget (fc, w);
 
-	gtk_object_set_data(GTK_OBJECT(fs->file_list),"frame",w);
-	gtk_signal_connect(GTK_OBJECT(fs->file_list),"select_row",
-			   GTK_SIGNAL_FUNC(setup_preview),NULL);
-	gtk_object_set_data(GTK_OBJECT(fs->selection_entry),"frame",w);
-	gtk_signal_connect(GTK_OBJECT(fs->selection_entry),"changed",
-			   GTK_SIGNAL_FUNC(setup_preview),NULL);
-}
+		g_signal_connect (G_OBJECT (fc), "update-preview",
+				  G_CALLBACK (setup_preview), fentry->fsw);
 
-static void
-drag_data_received (GtkWidget        *widget,
-		    GdkDragContext   *context,
-		    gint              x,
-		    gint              y,
-		    GtkSelectionData *selection_data,
-		    guint             info,
-		    guint32           time,
-		    GnomePixmapEntry *pentry)
-{
-	GList *uris, *li;
-	GnomeVFSURI *uri = NULL;
-	GtkWidget *entry;
-	
-	g_return_if_fail (pentry != NULL);
-	g_return_if_fail (GNOME_IS_PIXMAP_ENTRY (pentry));
+		/* the path can be already set */
+		if ((path = gtk_file_chooser_get_filename (fc)) != NULL)
+			setup_preview (NULL, fentry->fsw);
 
-	entry = gnome_pixmap_entry_gtk_entry (pentry);
+		g_free (path);
+	} else {
+		fs = GTK_FILE_SELECTION (fentry->fsw);
+		
+		/* we already got this */
+		if (g_object_get_data (G_OBJECT (fs->file_list), "frame") != NULL)
+			return;
 
-	/*here we extract the filenames from the URI-list we recieved*/
-	uris = gnome_vfs_uri_list_parse (selection_data->data);
+		hbox = fs->file_list;
+		do {
+			hbox = hbox->parent;
+			if(!hbox) {
+				g_warning(_("Can't find an hbox, using a normal file "
+					    "selection"));
+				return;
+			}
+		} while(!GTK_IS_HBOX(hbox));
 
-	/* FIXME: Support multiple files */
-	for (li = uris; li != NULL; li = li->next) {
-		uri = li->data;
-		/* FIXME: Support non-local files */
-		if (gnome_vfs_uri_is_local (uri)) {
-			break;
-		}
-		uri = NULL;
+		w = gtk_frame_new(_("Preview"));
+		gtk_widget_show(w);
+		gtk_box_pack_end(GTK_BOX(hbox),w,FALSE,FALSE,0);
+		gtk_widget_set_size_request (w, 110, 110);
+		g_object_set_data (G_OBJECT (w), "fs", fs);
+
+		g_object_set_data (G_OBJECT (fs->file_list), "frame", w);
+		g_signal_connect (G_OBJECT (gtk_tree_view_get_selection (GTK_TREE_VIEW (fs->file_list))), "changed",
+				  G_CALLBACK (setup_preview), fs->file_list);
+		g_object_set_data (G_OBJECT (fs->selection_entry), "frame", w);
+		g_signal_connect (fs->selection_entry, "changed",
+				  G_CALLBACK (setup_preview), fs->selection_entry);
 	}
-
-	/*if there's isn't a file*/
-	if (uri == NULL) {
-		gtk_drag_finish (context, FALSE, FALSE, time);
-		/*free the list of files we got*/
-		gnome_vfs_uri_list_free (uris);
-		return;
-	}
-
-	gtk_entry_set_text (GTK_ENTRY (entry),
-			    gnome_vfs_uri_get_path (uri));
-
-	/*free the list of files we got*/
-	gnome_vfs_uri_list_free (uris);
 }
 
 static void
@@ -469,7 +457,7 @@ drag_data_get  (GtkWidget          *widget,
 	g_free(file);
 	gtk_selection_data_set (selection_data,
 				selection_data->target,
-				8, string, strlen(string)+1);
+				8, (unsigned char *)string, strlen(string)+1);
 	g_free(string);
 }
 
@@ -478,21 +466,14 @@ turn_on_previewbox(GnomePixmapEntry *pentry)
 {
 	pentry->_priv->preview_sw = gtk_scrolled_window_new(NULL,NULL);
 	gtk_widget_ref(pentry->_priv->preview_sw);
-	gtk_drag_dest_set (GTK_WIDGET (pentry->_priv->preview_sw),
-			   GTK_DEST_DEFAULT_MOTION |
-			   GTK_DEST_DEFAULT_HIGHLIGHT |
-			   GTK_DEST_DEFAULT_DROP,
-			   drop_types, 1, GDK_ACTION_COPY);
-	gtk_signal_connect (GTK_OBJECT (pentry->_priv->preview_sw), "drag_data_received",
-			    GTK_SIGNAL_FUNC (drag_data_received),pentry);
 	/*for some reason we can't do this*/
-	/*gtk_signal_connect (GTK_OBJECT (pentry->_priv->preview_sw), "drag_data_get",
-			    GTK_SIGNAL_FUNC (drag_data_get),pentry);*/
+	/*g_signal_connect (G_OBJECT (pentry->_priv->preview_sw), "drag_data_get",
+			    G_CALLBACK (drag_data_get), pentry);*/
 	gtk_box_pack_start (GTK_BOX (pentry), pentry->_priv->preview_sw, TRUE, TRUE, 0);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(pentry->_priv->preview_sw),
 				       GTK_POLICY_AUTOMATIC,
 				       GTK_POLICY_AUTOMATIC);
-	gtk_widget_set_usize(pentry->_priv->preview_sw,100,100);
+	gtk_widget_set_size_request (pentry->_priv->preview_sw, 100, 100);
 	gtk_widget_show (pentry->_priv->preview_sw);
 
 	pentry->_priv->preview = gtk_label_new(_("No Image"));
@@ -515,16 +496,16 @@ gnome_pixmap_entry_instance_init (GnomePixmapEntry *pentry)
 
 	pentry->_priv = g_new0(GnomePixmapEntryPrivate, 1);
 
-	pentry->_priv->do_preview = 1;
+	pentry->_priv->do_preview = 0;
 
 	pentry->_priv->last_preview = NULL;
 
 	pentry->_priv->preview = NULL;
 	pentry->_priv->preview_sw = NULL;
 
-	gtk_signal_connect_after(GTK_OBJECT(pentry),"browse_clicked",
-				 GTK_SIGNAL_FUNC(browse_clicked),
-				 pentry);
+	g_signal_connect_after (pentry, "browse_clicked",
+				G_CALLBACK (browse_clicked),
+				pentry);
 
 	p = gnome_program_locate_file (NULL /* program */,
 				       GNOME_FILE_DOMAIN_PIXMAP,
@@ -535,9 +516,8 @@ gnome_pixmap_entry_instance_init (GnomePixmapEntry *pentry)
 	g_free(p);
 
 	w = gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(pentry));
-	gtk_signal_connect(GTK_OBJECT(w),"changed",
-			   GTK_SIGNAL_FUNC(entry_changed),
-			   pentry);
+	g_signal_connect (G_OBJECT (w), "changed",
+			  G_CALLBACK (entry_changed), pentry);
 
 	/*just in case there is a default that is an image*/
 	refresh_preview(pentry);
@@ -587,7 +567,7 @@ gnome_pixmap_entry_new (const gchar *history_id, const gchar *browse_dialog_titl
 {
 	GnomePixmapEntry *pentry;
 
-	pentry = gtk_type_new (GNOME_TYPE_PIXMAP_ENTRY);
+	pentry = g_object_new (GNOME_TYPE_PIXMAP_ENTRY, NULL);
 
 	gnome_pixmap_entry_construct (pentry, history_id, browse_dialog_title, do_preview);
 	return GTK_WIDGET (pentry);
@@ -695,8 +675,6 @@ gnome_pixmap_entry_preview_widget(GnomePixmapEntry *pentry)
  * Description: Sets the default path for the file entry. The new
  * subdirectory should be specified relative to the default GNOME
  * pixmap directory.
- *
- * Returns:
  **/
 void
 gnome_pixmap_entry_set_pixmap_subdir(GnomePixmapEntry *pentry,
@@ -727,8 +705,6 @@ gnome_pixmap_entry_set_pixmap_subdir(GnomePixmapEntry *pentry,
  * the entry.  If the preview is on, we also load the files and check
  * for them being real images.  If it is off, we don't check files
  * to be real image files.
- * 
- * Returns:
  **/
 void
 gnome_pixmap_entry_set_preview (GnomePixmapEntry *pentry, gboolean do_preview)
@@ -766,8 +742,6 @@ gnome_pixmap_entry_set_preview (GnomePixmapEntry *pentry, gboolean do_preview)
  *
  * Description: Sets the minimum size of the preview frame in pixels.
  * This works only if the preview is enabled.
- *
- * Returns:
  **/
 void
 gnome_pixmap_entry_set_preview_size(GnomePixmapEntry *pentry,
@@ -779,8 +753,8 @@ gnome_pixmap_entry_set_preview_size(GnomePixmapEntry *pentry,
 	g_return_if_fail (preview_w>=0 && preview_h>=0);
 
 	if(pentry->_priv->preview_sw)
-		gtk_widget_set_usize(pentry->_priv->preview_sw,
-				     preview_w,preview_h);
+		gtk_widget_set_size_request (pentry->_priv->preview_sw,
+					     preview_w, preview_h);
 }
 
 /* Ensures that a pixmap entry is not in the waiting list for a preview update.  */
@@ -814,7 +788,7 @@ ensure_update (GnomePixmapEntry *pentry)
  * successfully loaded if preview is disabled.  If the preview is
  * disabled, the file is only checked if it exists or not.
  *
- * Returns: Newly allocated string containing path, or %NULL on error. 
+ * Returns: Newly allocated string containing path, or %NULL on error.
  **/
 char *
 gnome_pixmap_entry_get_filename(GnomePixmapEntry *pentry)
